#!/bin/bash
# (C) Klaus Knopper Nov 2002
# Calls scanpartitions as root and adds entries to /etc/fstab

PATH="/bin:/sbin:/usr/bin:/usr/sbin"
export PATH
umask 022

force=""
verbose=""
remove=""
mkicons=""
user=""
group=""
mpowner="root"
mpgroup="disk"
arg="$1"
while [ -n "$arg" ]; do
 case "$arg" in
  -f*) force="yes" ;;
  -v*) verbose="yes" ;;
  -r*) remove="yes" ;;
  -i*) mkicons="yes" ;;
  -u*) shift; user="$1"; mpowner="$1" ;;
  -g*) shift; group="$1"; mpgroup="$1" ;;
  *) echo "Usage: $0 [-f[orce]] [-v[erbose]] [-r[emove_old]] [-i[cons]] [-u[ser] uid] [-g[roup] gid]" ;;
 esac
 shift
 arg="$1"
done

case "$(cat /proc/cmdline)" in *\ nofstab)
 if [ -z "$force" ]; then
  [ -n "$verbose" ] && echo "$0: nofstab bootoption detected, exiting." >&2
  exit 0
 fi
 ;;
esac

[ ! -r /proc/partitions ] && { echo "$0: /proc not mounted, exiting" >&2; exit 1; }

MYPID="$$"

# Check if it's already running, wait for completion
while [ -r /var/run/rebuildfstab.pid ]; do
 sleep 3
 read PID </var/run/rebuildfstab.pid 2>/dev/null # Get PID
 if [ -n "$PID" ]; then
  test -d /proc/"$PID" >/dev/null 2>&1 && continue
  break
 fi
done

# Lock.
echo "$MYPID" >/var/run/rebuildfstab.pid

TMP="/tmp/fstab.$$.tmp"
ADDEDBYKNOPPIX="# Added by KNOPPIX"

# Simple shell grep, searches for lines STARTING with string
stringinfile(){
 while read line; do
  case "$line" in $1*) return 0;; esac
 done <"$2"
 return 1
}

removeentries(){
 # Remove comment line $1 and the following line from file $2
 # sed '/^# Added by KNOPPIX/{N;d;}'
 while read line; do
  case "$line" in $1) read line; continue ;; esac
  echo "$line"
 done <"$2"
}

[ -n "$verbose" ] && echo "Scanning for new harddisks/partitions..." >&2
rm -f "$TMP"

if [ -n "$remove" ]; then
 removeentries "$ADDEDBYKNOPPIX" /etc/fstab >"$TMP"
else
 cat /etc/fstab >"$TMP"
fi

count=0
while read device mountpoint fstype relax; do
 stringinfile "$device " "$TMP" || \
 { count="$((count + 1))"
   if [ -n "$mountpoint" -a "$mountpoint" != "none" -a ! -d "$mountpoint" ]; then
    mkdir -p "$mountpoint" 2>/dev/null
    chown "$mpowner"."$mpgroup" "$mountpoint" 2>/dev/null
    chmod 775 "$mountpoint" 2>/dev/null
    compat="/mnt/${mountpoint##*/}"
    [ -d "$compat" -o -L "$compat" ] || ln -sf "$mountpoint" "$compat" 2>/dev/null
   fi
   options="noauto,users,exec"
   case "$fstype" in
    vfat)  options="${options},umask=000,shortname=winnt" ;;
    msdos) options="${options},umask=000" ;;
    swap)  options="defaults" ;;
    ntfs*) options="${options},umask=000,force";;
   esac
   case "$fstype" in
   ntfs*|vfat|msdos)
   [ -n "$user" ] && options="$options,uid=$user"
   [ -n "$group" ] && options="$options,gid=$group"
   ;;
   esac
   echo "$ADDEDBYKNOPPIX"
   echo "$device $mountpoint $fstype $options 0 0"; }
done >>"$TMP" <<EOT
$(scanpartitions)
EOT

# Add CD/DVD-Roms
cdcount=0
while read drive name drives; do
 case "$drive$name" in drivename:)
  for d in $drives; do
   mountpoint="/media/$d"
   if [ -n "$mountpoint" -a "$mountpoint" != "none" -a ! -d "$mountpoint" ]; then
    mkdir -p "$mountpoint" 2>/dev/null
    compat="/mnt/${mountpoint##*/}"
    [ -d "$compat" -o -L "$compat" ] || ln -sf "$mountpoint" "$compat" 2>/dev/null
   fi
   stringinfile "/dev/$d " "$TMP" || \
    { cdcount="$((cdcount + 1))"
      echo "$ADDEDBYKNOPPIX"
      echo "/dev/$d $mountpoint auto ro,noauto,users,exec 0 0"; }
  done
  break
  ;;
 esac
done >>"$TMP" <<EOT
$(cat /proc/sys/dev/cdrom/info 2>/dev/null)
EOT

[ -n "$verbose" ] && { [ "$count" -gt 0 -o "$cdcount" -gt 0 ] && echo "Adding $count new partitions and $cdcount new cdrom-type devices to /etc/fstab." >&2 || echo "No new partitions found." >&2; }
mv -f "$TMP" /etc/fstab

[ -n "$mkicons" ] && type -p mkdesktophdicons  >/dev/null 2>&1 && mkdesktophdicons >/dev/null 2>&1

rm -f /var/run/rebuildfstab.pid

[ -n "$DISPLAY" ] && sleep 1

exit 0
